#include "main.h"

#include "CodeData.h"
#include "IR_Rimo.h"
#include "Motion.h"


#case    // Case sensitive

#define __16F88
#define __LIMIT    // ~bgXCb`̎gp錾
// --------------------------------------------------------------


#ifdef __16F876
    #include <16f876.h>
    #fuses HS, NOWDT, PUT, NOBROWNOUT, NOLVP, NOCPD, NOWRT, NOPROTECT
    
#else #ifdef __16F88
    #include <16f88.h>
    #fuses HS, NOWDT, NOPROTECT, PUT, NOBROWNOUT, NOLVP, NOCPD, MCLR
    
#else #ifdef __16F84A
    #include <16f84A.h>
    #fuses HS, NOWDT, PUT, NOPROTECT
#else
    #error This device in not supported!!
#endif
#endif
#endif


#device ADC=10 *=16
#use delay(Clock = 20000000)

#use fast_io(A)
#use fast_io(B)


// *************************************************************************

#define    MOTOR_LEFT1     PIN_B1
#define    MOTOR_LEFT2     PIN_B2
#define    MOTOR_RIGHT1    PIN_B3
#define    MOTOR_RIGHT2    PIN_B4

#define    MOTOR_ARM1  PIN_B5
#define    MOTOR_ARM2  PIN_B6

#define    LIMIT_UP    PIN_A1
#define    LIMIT_DOWN  PIN_A0

#define    LED         PIN_A2

#define    DRIVE_TIME  200 // [^쎞 [ msec]

#define    STOP    0
#define    GO      1
#define    BACK    2
#define    RIGHT   3
#define    LEFT    4
#define    UP      5
#define    DOWN    6

#define    IR_LED  PIN_A2    // MmFpkdc

// *************************************************************************
int direction = STOP;    // A[p[^̓


/* =====================================================================
    C֐
===================================================================== */
void main()
{
#ifdef __LIMIT
    set_tris_a(0b00011011);    
#else
    set_tris_a(0b00000011);
#endif
    output_a(0x00);
    
    set_tris_b(0b00000001);
    output_b(0x00);

#ifndef __16F84A
    setup_adc_ports(NO_ANALOGS);    // Set AD Converter
    setup_adc(ADC_OFF);
#endif

    // ------------------------------------------------------

    MotorTire(STOP);
    MotorArm (STOP);

    InitIR();
    
    enable_interrupts(GLOBAL);
    FlashLED(2);
                
    while(1)
    {
        if(IR_ReceiveIF) ReceiveFromRC();
        
#ifdef __LIMIT
        CheckArmLimit();
#endif
    }
}

// =====================================================================
// M(ԊOR)
void ReceiveFromRC()
{
    IR_ReceiveIF = FALSE;
    
    switch(IR_ReceiveData)
    {
        case RC_CODE_02:        MotorTire(GO);     break;
        case RC_CODE_08:        MotorTire(BACK);   break;
        case RC_CODE_05:        MotorTire(STOP);   break;
        case RC_CODE_04:        MotorTire(LEFT);   break;
        case RC_CODE_06:        MotorTire(RIGHT);  break;
#ifdef __LIMIT
        case RC_CODE_ChUP:      MotorArm(UP);      break;
        case RC_CODE_ChDOWN:    MotorArm(DOWN);    break;
#endif
        case RC_CODE_VolUP:     MotorArmTime(UP);  break;
        case RC_CODE_VolDOWN:   MotorArmTime(DOWN);break;
                
        case RC_CODE_10:        Motion(1);         break;
        case RC_CODE_11:        Motion(2);         break;
        case RC_CODE_12:        Motion(3);         break;
        
        default:
            break;
    }
}

// =====================================================================

// ^Cp[^쓮
void MotorTire(int select)
{
    int leftOut1, leftOut2, rightOut1, rightOut2;
    switch(select)
    {
        case STOP:  leftOut1 = 0; leftOut2 = 0; rightOut1 = 0; rightOut2 = 0; break;
        case GO:    leftOut1 = 0; leftOut2 = 1; rightOut1 = 0; rightOut2 = 1; break;
        case BACK:  leftOut1 = 1; leftOut2 = 0; rightOut1 = 1; rightOut2 = 0; break;
        case RIGHT: leftOut1 = 0; leftOut2 = 1; rightOut1 = 1; rightOut2 = 0; break;
        case LEFT:  leftOut1 = 1; leftOut2 = 0; rightOut1 = 0; rightOut2 = 1; break;      

        default:
            return;        
    }
    
    // [^֏o
    output_bit(MOTOR_LEFT1,  leftOut1 );
    output_bit(MOTOR_LEFT2,  leftOut2 );
    output_bit(MOTOR_RIGHT1, rightOut1);
    output_bit(MOTOR_RIGHT2, rightOut2);
}    

// ^Cp̉E[^쓮
void Right(int select)
{
    switch(select)
    {
        case 0:    output_bit(MOTOR_RIGHT1, 0);    output_bit(MOTOR_RIGHT2, 0); break;  // ~
        case 1:    output_bit(MOTOR_RIGHT1, 0);    output_bit(MOTOR_RIGHT2, 1); break;  // ]
        case 2:    output_bit(MOTOR_RIGHT1, 1);    output_bit(MOTOR_RIGHT2, 0); break;  // t]
        
        default:
            break;
    }
}

// ^Cp̍[^쓮
void Left(int select)
{
    switch(select)
    {
        case 0:    output_bit(MOTOR_LEFT1, 0); output_bit(MOTOR_LEFT2, 0); break;  // ~
        case 1:    output_bit(MOTOR_LEFT1, 0); output_bit(MOTOR_LEFT2, 1); break;  // ]
        case 2:    output_bit(MOTOR_LEFT1, 1); output_bit(MOTOR_LEFT2, 0); break;  // t]

        default:
            break;
    }
}

// A[p[^쓮
void MotorArm(int select)
{
    int out1, out2;
    switch(select)
    {
        case STOP:  out1 = 0; out2 = 0; break;
        case UP:    out1 = 1; out2 = 0; break;
        case DOWN:  out1 = 0; out2 = 1; break;
        
        default:
            return;        
    }
    
    // [^֏o
    output_bit(MOTOR_ARM1, out1);
    output_bit(MOTOR_ARM2, out2);
    
    direction = select;
}

void Arm(int select)
{
    switch(select)
    {
        case 0:    MotorArm(STOP); break;
        case 1:    MotorArm(UP  ); break;
        case 2:    MotorArm(DOWN); break;
        
        default:
            return;        
    }
}

// A[p[^莞ԋ쓮
void MotorArmTime(int select)
{
    long i;
    MotorArm(select);
    
    for(i=0; i<DRIVE_TIME; i++)
    {
        
#ifdef __LIMIT
        CheckArmLimit();
        if(direction == STOP) break;
#endif
        delay_ms(1);
    }
    MotorArm(STOP);
}

// w莞Ԃ̒x ( time[sec] )
void Time(float time)
{
    int i;
    int num;
    
    // 100msec wl10{񐔂JԂ
    // ex) time = 1.0 Ȃ 100msec x 10 = 1.0sec
    num = (int)(time * 10);
    
    for(i=0; i<num; i++)
    {
        if(IR_ReceiveIF) break;
        
#ifdef __LIMIT
        CheckArmLimit();
#endif    
        delay_ms(100);
    }
}

// A[̏㉺`FbN
void CheckArmLimit()
{
    if(!input(LIMIT_UP  ) && direction == UP)
    {             
        delay_us(10);
        if(!input(LIMIT_UP  )) MotorArm(STOP);
    }
        
    if(!input(LIMIT_DOWN) && direction == DOWN)
    {
        delay_us(10);
        if(!input(LIMIT_DOWN)) MotorArm(STOP);
    }
}

// =====================================================================

// kdc_
void FlashLED(int num)
{
    num <<= 1;
    for(; num; num--)
    {
        output_bit(IR_LED, num & 0x01);
        delay_ms(200);
    }

    output_low(IR_LED);
}
